package com.tzld.longarticle.recommend.server.service.rank;


import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.score.ScoreService;
import com.tzld.longarticle.recommend.server.service.score.ScoreStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ContentPoolStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.SimilarityStrategy;
import com.tzld.longarticle.recommend.server.service.score.strategy.ViewCountStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService {

    @Autowired
    private ScoreService scoreService;

    public RankResult rank(RankParam param) {

        ScoreResult result = scoreService.score(convertToScoreParam(param));
        Map<String, Map<Class<? extends ScoreStrategy>, Double>> scoreMap = result.getScoreMap();

        List<RankItem> items = CommonCollectionUtils.toList(param.getContents(), c -> {
            Map<Class<? extends ScoreStrategy>, Double> map = scoreMap.get(c.getId());
            double score = 1 * map.getOrDefault(SimilarityStrategy.class, 0.0)
                    + 10 * map.getOrDefault(ViewCountStrategy.class, 0.0)
                    + 100 * map.getOrDefault(ContentPoolStrategy.class, 0.0);

            RankItem item = new RankItem();
            item.setContent(c);
            item.setScore(score);
            return item;

        });
        Collections.sort(items, Comparator.comparingDouble(o -> -o.getScore()));


        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);
        // 1 相似去重
        contents = deduplication(contents);
        // 2 文章分组


        // 3 按位置选文章
        // 头条

        // 次条

        // 3-8


        return new RankResult(items);
    }

    private ScoreParam convertToScoreParam(RankParam param) {
        ScoreParam scoreParam = new ScoreParam();
        scoreParam.setAccountName(param.getAccountName());
        scoreParam.setContents(param.getContents());
        return scoreParam;
    }

    private List<Content> deduplication(List<Content> contents) {
        List<String> titles = new ArrayList<>();
        List<Content> result = new ArrayList<>();
        // 遍历所有列表
        for (Content c : contents) {
            if (similarity(c.getTitle(), titles)) {
                continue;
            } else {
                result.add(c);
                titles.add(c.getTitle());
            }
        }

        return result;
    }

    private boolean similarity(String title, List<String> titles) {
        return TitleSimilarCheckUtil.isDuplicateContent(title, titles);
    }

}
